/* 	libchenabler - API definition of the call history service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
  *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

/**
 *@file the Voice Call History interface
 */


#ifndef ___LIPS_VOCH_HISTORY_H
#define ___LIPS_VOCH_HISTORY_H

#include "vochtype.h"
#include "vocherror.h"
#include "vochcallhistory.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */



  void voch_init (void);

  void voch_quit (void);




/**
 *@brief		connect to an history record and return the connected history record handle if connected successfully.
 *	applications could repeatedly call this function to connect to all the history record in the device.
 *@param		index		the index of the targeted history. All history in the device are indexed
 *									from 0 to n
 *@param		sid			returned value, the handle of the connected history in the current session
 *@return	0											if connect successfully
 *					VOCH_ERROR_INVALID			there is no address book indexed with index
 *					VOCH_ERROR_OUT_OF_RANGE	There is no more address book in the current device
 */

  voch_err_t voch_connect (uint32_t index, voch_sid_t *sid);

/**
 *@brief		disconnect history record with identifier sid
 *@param		sid				the handle of the connected history in the current session
 */
  voch_err_t voch_disconnect (voch_sid_t sid);


/**
 *@brief		create a new history item
 *@param		sid				the handle of the connected history in the current session
 *@param		history		a newly created history instance
 *@note		applications should later free the returned contact.
 */
  voch_err_t voch_create_history (voch_sid_t sid,
                                  voch_callhistory_t **history);


/**
 *@brief		add a new history instance to the history
 *@param		sid				the handle of the connected history in the current session.
 *@param		contact		pointer to the history instance to be added
 */
  voch_err_t voch_add_history (voch_sid_t sid, voch_callhistory_t *history);


/**
 *@brief		remove a history item from the history
 *@param		sid				the handle of the connected history record in the current session.
 *@param		uid				identifer of an history instance
 */
  voch_err_t voch_rm_history (voch_sid_t sid, const voch_cid_t *cid);

/**
 *@brief		remove several history items from history
 *@param		sid				the handle of the connected history in the current session.
 *@param		cids			a zero terminated array, containing a set of history identifiers which will be deleted from
 *									the history */
  voch_err_t voch_rm_historys (voch_sid_t sid, const voch_cid_t **cids);

/**
 *@brief		update a history
 *@param		sid				the handle of the connected history record in the current session
 *@param		history		pointer to the history to be updated
 *@param		fids			ab_FIELD_ID_INVALID terminated array, which explicitly specifies which fields should be
 *									updated. if fids set to NULL, the function itself should figure out which fields should be
 *									updated.
 */
  voch_err_t voch_update_history (voch_sid_t sid,
                                  voch_callhistory_t *history,
                                  voch_fid_t *fids);


/**
 *@brief		get a history instance from history record by its id
 *@param		sid				the handle of the history record in the current session.
 *@param		cid				identifer of the history item to be retrieved
 *@param		history		history instance returned from the history record
 *@note		apps should later free the returned history instance
 */
  voch_err_t voch_get_history (voch_sid_t sid, voch_cid_t cid,
                               voch_callhistory_t **history);

/**
 *@brief		query historys from a history record which match some querying criteria.
 *@param		sid				the handle of the connected history record in the current session
 *@param		match			a history instance containg querying criteria,if set to NULL,then get all the contacts
  *@note		apps should later free the returned array
 */
  voch_cid_t **voch_get_history_cids (voch_sid_t sid,
                                      const voch_callhistory_t *match);


/**
 *@brief		query history items which have a specific number.
 *@param		sid				the handle of the connected history record in the current session
 *@param		number		a phone number. it could be a mobile number, home number, office number and so on
 *@param		iter			a zero terminated array, containing a set of contact identifiers which match the criteria
 *@note		apps should later free the returned array
 */
  voch_cid_t **voch_get_historys_by_number (voch_sid_t sid, const char *number);

/**
 *@brief		query the number of history items in the history record.
 *@param		sid				the handle of the connected history record in the current session
 *@param		type		a phone number. it could be a mobile number, home number, office number and so on
 *@param		count			a pointer to the number of items
 *@note		apps should later free the returned number
 */
  voch_err_t voch_get_history_count (voch_sid_t sid, voch_call_type_t type,
                                     uint32_t *count);


/**
 *Advanced search
 */

/**
 *@brief 	query historys from an history record which match some querying criteria.
 *@param 	sid				the handle of the connected history record in the current session
 *@param		match			a historyt instance containing querying criteria,if set to NULL,then get all the historys
 *@return	An array of historyss terminated by NULL, created by enabler and should be freed by app
 *@note		apps should later free the iterator
 */

  voch_callhistory_t **voch_get_historys (voch_sid_t sid,
                                          const voch_callhistory_t *match);

/**
 *@brief		query historys which have a specific type.
 *@param		sid				the handle of the connected history in the current session
 *@param		type		a call type.
 *@param		iter			a zero terminated array, containing a set of history identifiers which match the criteria
 *@note		apps should later free the returned array
 */
  voch_callhistory_t **voch_get_historys_by_type (voch_sid_t sid,
                                                  voch_call_type_t type);

/**
 *@brief		get all the field templates contained in the voc history template
 *@param		sid				the handle of the connected history record in the current session
 *@param		ftids			voch_TEMPLATE_ID_INVALID terminated fid array, allocated by enabler and freed by app.
 */
  voch_fid_t **voch_get_field_templates (voch_sid_t sid);

#ifdef __cplusplus
}
#endif                          /* __cplusplus */
#endif                          /* _LIPS_VOCH_HISTORY_H */

