/* 	libchenabler - API definition of the call history service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <sqlite3.h>

#include <config.h>
#include "callhistory.h"

#include "vochtype.h"
#include <uuid/uuid.h>

#include <template.h>
#include <record.h>
#include <database.h>

#include "callhistory-priv.h"
#include "vochcallhistory-priv.h"
#include "vochcallhistory.h"
#include "env.h"

#define VOCHISTORY_TEMPLATE_FILE                                     "./vochistory.templ"

#define VOCHISTORY_ADMIN_DB_DISCOVERY_SQL        "select *from  vochistory;"
#define SESSION_RANGE           100
static GPtrArray *vochlists = NULL;
static GHashTable *session_maps = NULL;
static FieldTemplate *history_template = NULL;  //It contains a pointer to a hashtable, GHashTable *fdescs


/*Create new history info item */
static VocHistoryInfo *
voch_info_new (void)
{
  VocHistoryInfo *info = NULL;

  info = g_new0 (VocHistoryInfo, 1);
  if (info == NULL)
  {
    return NULL;
  }
  return info;
}


/*Free the history info item */
static void
voch_info_free (VocHistoryInfo *info)
{
  if (info == NULL)
  {
    return;
  }
  if (info->source)
  {
    g_free (info->source);
  }
  g_free (info);
  return;
}


/*print the information */
void
voch_info_print (const VocHistoryInfo *info)
{
  g_print ("VocHistoryInfo[0x%08x]\n", (guint32) info);
  g_print ("|ID     : %d\n", info->id);
  g_print ("|SOURCE : %s\n", info->source);
  g_print ("|REF    : %d\n", info->ref);
  return;
}


/*find template, and store the information into the static variable : history_template */
voch_err_t
voch_plugin_init (void)
{

/*get template file name */
  if (history_template == NULL)
  {
    gchar *templ_fname = NULL;

    templ_fname = voch_env_get_history_template_file ();
    if (templ_fname == NULL)
    {
      return VOCH_ERR_TEMPLATE_PARSE;
    }
//create a new filed template        
    history_template = field_template_new ();

//parse the template    
    if (field_template_parse_file (history_template, templ_fname) != TRUE)
    {
      g_print ("%s(): template parse error.[%s]\n", __FUNCTION__, templ_fname);
      g_free (templ_fname);
      return VOCH_ERR_TEMPLATE_PARSE;
    }
    g_free (templ_fname);
  }
  return VOCH_ERR_NONE;
}


/*free template */
voch_err_t
voch_plugin_destroy (void)
{
  if (history_template)
  {
    field_template_free (history_template);
  }
  return VOCH_ERR_NONE;
}


/*Add info to the vochlists array, now , it only contains one item.id|type|source:1|1|vochistory.db */
static int
voch_discover_cb (void *unused, int argc, char **argv, char **azColName)
{
  if (argc == 3)
  {
    VocHistoryInfo *info = voch_info_new ();

    info->id = atoi (argv[1]);  // argc: num of column; argv: name of column;
    info->source = g_strdup (argv[2]);
    info->ref = 0;
    g_ptr_array_add (vochlists, (gpointer) info);
  }
  return 0;
}


/*get record from the database */
static voch_err_t
voch_discover_tables (const gchar *adim_fname)
{
  sqlite3 *database = NULL;
  gchar *err_str = NULL;
  int ret;

  if (!g_file_test (adim_fname, G_FILE_TEST_IS_REGULAR))
  {
    g_print ("%s(): admin database lost.\n", __FUNCTION__);
    return VOCH_ERR_DB_LOST;
  }
  if (sqlite3_open (adim_fname, &database))     // open admin.db
  {
    g_print ("%s(): open admin database failed. %s\n", __FUNCTION__,
             sqlite3_errmsg (database));
    sqlite3_close (database);
    return VOCH_ERR_DB_TRANS;
  }
/*callback for each item in the vochistory table */
  ret =
    sqlite3_exec (database, VOCHISTORY_ADMIN_DB_DISCOVERY_SQL,
                  voch_discover_cb, NULL, &err_str);
  if (ret != SQLITE_OK)
  {
    g_print ("%s(): discovery failed. [%s]\n", __FUNCTION__, err_str);
    g_free (err_str);
    return VOCH_ERR_DB_TRANS;
  }
  sqlite3_close (database);
  return VOCH_ERR_NONE;
}


/* initialize info->priv to point to the vochistoy.db */
static voch_err_t
voch_initialize (VocHistoryInfo *info)
{
  VocHistory *priv = NULL;
  gchar *db_path = NULL;

  g_assert (info);
  g_print ("%s()\n", __FUNCTION__);
  priv = g_new0 (VocHistory, 1);
  if (priv == NULL)
  {
    return VOCH_ERR_MEM_ALLOC;
  }
  db_path =
    g_build_filename (voch_env_get_database_prefix (), info->source, NULL);
  g_print ("%s():%s\n", __FUNCTION__, info->source);

/*create the priv->history_db, Create Database tables with the template */
  priv->history_db =
    (gpointer) record_db_new (db_path, "vochistory", history_template);
  if (priv->history_db == NULL)
  {
    g_print ("%s(): database error.[%s]\n", __FUNCTION__, db_path);
    g_free (priv);
    return VOCH_ERR_DB_FAILURE;
  }
  if (record_db_open (priv->history_db) != DB_ERROR_NONE)
  {
    g_print ("%s(): database open error.\n", __FUNCTION__);
    record_db_free (priv->history_db);
    g_free (priv);
    return VOCH_ERR_DB_FAILURE;
  }
/* give value to info->priv */
  info->priv = (gpointer) priv;
  return VOCH_ERR_NONE;
}


/* free */
static voch_err_t
voch_finalize (VocHistoryInfo *info)
{
  VocHistory *priv = NULL;

  g_assert (info);
  g_print ("%s()\n", __FUNCTION__);
  priv = (VocHistory *) info->priv;
  if (priv->history_db)
  {
    if (record_db_close (priv->history_db) != DB_ERROR_NONE)
    {
      record_db_free (priv->history_db);
      g_free (priv);
      return VOCH_ERR_DB_FAILURE;
    }
    record_db_free (priv->history_db);
  }
  g_free (priv);
  return VOCH_ERR_NONE;
}


/* get the vochistory.db and put into info->priv */
static voch_err_t
voch_module_initialize (void)
{
  gchar *db_fname = NULL;
  voch_err_t ret;

  g_print ("%s():\n", __FUNCTION__);

  /* The following 3 steps initialize the 3 static variables. */
  vochlists = g_ptr_array_new ();
  session_maps =
    g_hash_table_new_full (g_int_hash, g_int_equal, g_free, g_free);

  /* put the contents in the template into the Fieldtemplate:history_template->desc    */
  voch_plugin_init ();

  /* get file name of dababase */
  db_fname = voch_env_get_admin_database ();    // get the admin DB: admin.db

  /* read each item into the admin.db, the item is translated into VocHistoryInfo and put into the array: vochlists */
  ret = voch_discover_tables (db_fname);
  if (ret != VOCH_ERR_NONE)
  {
    g_free (db_fname);
    return ret;
  }
  g_free (db_fname);

  /* get the first info, which corresponding to the first item in database         */
  VocHistoryInfo *info = (VocHistoryInfo *) g_ptr_array_index (vochlists, 0);

  voch_initialize (info);
  voch_info_print (info);
  return VOCH_ERR_NONE;
}

static void
voch_module_finalize (void)
{
  int i;
  int count;

  g_print ("%s():\n", __FUNCTION__);
  g_hash_table_destroy (session_maps);
  count = vochlists->len;
  for (i = 0; i < count; i++)
  {
    VocHistoryInfo *info = NULL;

    info = g_ptr_array_remove_index_fast (vochlists, 0);
    voch_finalize (info);
    voch_info_free (info);
  }
  g_ptr_array_free (vochlists, TRUE);
  voch_plugin_destroy ();
  return;
}


/* uuid===cid */
static gpointer
voch_generate_cid (void)
{
  voch_cid_t cid;
  gchar *cid_str;

  cid_str = g_new0 (gchar, sizeof (voch_cid_t));
  uuid_generate (cid);
  memcpy (cid_str, cid, sizeof (voch_cid_t));
  return cid_str;
}


/* initialize the voc history by calling voch_module_initialize() */
void
voch_init (void)
{
  if (voch_module_initialize () != VOCH_ERR_NONE)
  {
    g_print ("%s(): initialize failed.\n", __FUNCTION__);
    return;
  }
  return;
}


/* finalize */
void
voch_quit (void)
{
  voch_module_finalize ();
  return;
}


/* index: index in the admin.db which corresponding to the vochistory datebase */
//open the database file "vochistory.db" and get ready to querying and modifying the database

voch_err_t
voch_connect (uint32_t index, voch_sid_t *sid)
{
  VocHistoryInfo *info = NULL;
  guint32 *svalue = NULL;
  guint32 *ivalue = NULL;

  if (sid == NULL)
  {
    return VOCH_ERR_COMMON_NULL;
  }
  if (index < 0)
  {
    g_print ("%s(): [%d] is out of range .\n", __FUNCTION__, index);
    return VOCH_ERR_DB_OUT_RANGE;
  }
  /* info->priv stores the  pointer to the VocHistory (the Database tables created with the template) */
  info = (VocHistoryInfo *) g_ptr_array_index (vochlists, index);

  /* SESSION_RANGE:one index---multiple session?, reference time */
  *sid = index *SESSION_RANGE + info->ref;
  info->ref++;
  svalue = g_new0 (guint32, 1);
  *svalue = *sid;
  ivalue = g_new (guint32, 1);
  *ivalue = index;
  g_print ("%s(): sid = %d, index = %d\n", __FUNCTION__, *sid, *ivalue);

/* the pair of sid--index */
  g_hash_table_replace (session_maps, (gpointer) svalue, (gpointer) ivalue);
  return VOCH_ERR_NONE;
}

voch_err_t
voch_disconnect (voch_sid_t sid)
{
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);
  if (index)
  {
    VocHistoryInfo *info =
      (VocHistoryInfo *) g_ptr_array_index (vochlists, *index);
    info->ref--;
    g_hash_table_remove (session_maps, &sid);
  }
  return VOCH_ERR_NONE;
}

//Create a "voch_callhistory_t" with no data;
voch_err_t
voch_create_history (voch_sid_t sid, voch_callhistory_t **history)
{
  VocHistoryInfo *info = NULL;
  guint32 *index = NULL;
  VocHistory *priv = NULL;

  g_assert (history);
  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);
  if (index)
  {
    info = (VocHistoryInfo *) g_ptr_array_index (vochlists, *index);
  }
  priv = (VocHistory *) info->priv;
  *history = voch_callhistory_new (record_db_get_template (priv->history_db));
  if (*history == NULL)
  {
    return VOCH_ERR_MEM_ALLOC;
  }
  return VOCH_ERR_NONE;
}

voch_err_t
voch_add_history (voch_sid_t sid, voch_callhistory_t *history)
{
  VocHistoryInfo *info = NULL;
  guint32 *index = NULL;
  VocHistory *priv = NULL;
  gpointer cid = NULL;

  g_assert (history);           
  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);
  if (index)
  {
    info = (VocHistoryInfo *) g_ptr_array_index (vochlists, *index);
  }
  priv = (VocHistory *) info->priv;
  cid = voch_generate_cid ();
  record_set_single_field (history->record, HISTORY_CID, cid);
  g_free (cid);
  voch_callhistory_print (history);
  if (record_db_add_record (priv->history_db, history->record) != DB_ERROR_NONE)
  {
    g_print ("%s(): record add failed.\n", __FUNCTION__);
    return VOCH_ERR_DB_FAILURE;
  }
  return VOCH_ERR_NONE;
}

voch_err_t
voch_rm_history (voch_sid_t sid, const voch_cid_t *cid)
{
  VocHistoryInfo *info = NULL;
  guint32 *index = NULL;
  VocHistory *priv = NULL;

  g_assert (cid);
  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);
  if (index)
  {
    info = (VocHistoryInfo *) g_ptr_array_index (vochlists, *index);
  }
  g_assert (info);
  priv = (VocHistory *) info->priv;
  if (record_db_remove_record (priv->history_db, *cid) != DB_ERROR_NONE)
  {
    g_print ("%s(): record remove failed.\n", __FUNCTION__);
    return VOCH_ERR_DB_FAILURE;
  }
  return VOCH_ERR_NONE;
}

voch_err_t
voch_rm_historys (voch_sid_t sid, const voch_cid_t **cids)
{
  VocHistoryInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);
  if (index)
  {
    info = (VocHistoryInfo *) g_ptr_array_index (vochlists, *index);
  }
  g_assert (info);
  if (cids)
  {
    gint i;

    for (i = 0; cids[i] != NULL; i++)
    {
      VocHistory *priv;

      priv = (VocHistory *) info->priv;
      if (record_db_remove_record (priv->history_db, cids[i]) != DB_ERROR_NONE)
      {
        g_print ("%s(): record remove failed.\n", __FUNCTION__);
        return VOCH_ERR_DB_FAILURE;
      }
    }
  }

  else
  {
    VocHistory *priv;

    priv = (VocHistory *) info->priv;
    if (record_db_remove_records (priv->history_db, 0, NULL) != DB_ERROR_NONE)
    {
      g_print ("%s(): record remove failed.\n", __FUNCTION__);
      return VOCH_ERR_DB_FAILURE;
    }
  }
  return VOCH_ERR_NONE;
}

voch_err_t
voch_update_history (voch_sid_t sid, voch_callhistory_t *history,
                     voch_fid_t *fids)
{
  VocHistoryInfo *info = NULL;
  guint32 *index = NULL;
  VocHistory *priv = NULL;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);
  if (index)
  {
    info = (VocHistoryInfo *) g_ptr_array_index (vochlists, *index);
  }
  g_assert (info);
  priv = (VocHistory *) info->priv;
  int i;

  for (i = 0; fids && fids[i] != VOC_FIELD_ID_INVALID; i++)
  {
  }
  if (record_db_update_record (priv->history_db, history->record, i, fids) !=
      DB_ERROR_NONE)
  {
    g_print ("%s(): record update failed.\n", __FUNCTION__);
    return VOCH_ERR_DB_FAILURE;
  }
  return VOCH_ERR_NONE;
}

voch_err_t
voch_get_history (voch_sid_t sid, voch_cid_t cid, voch_callhistory_t **history)
{
  VocHistoryInfo *info = NULL;
  guint32 *index = NULL;
  VocHistory *priv = NULL;

  g_assert (history);
  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);
  if (index)
  {
    info = (VocHistoryInfo *) g_ptr_array_index (vochlists, *index);
  }
  priv = (VocHistory *) info->priv;
  *history = voch_callhistory_new (record_db_get_template (priv->history_db));
  if (*history == NULL)
  {
    return VOCH_ERR_MEM_ALLOC;
  }
  if (record_db_get_record (priv->history_db, (*history)->record, cid) !=
      DB_ERROR_NONE)
  {
    g_print ("%s(): record get failed.\n", __FUNCTION__);
    return VOCH_ERR_DB_FAILURE;
  }
  return VOCH_ERR_NONE;
}

voch_cid_t **
voch_get_history_cids (voch_sid_t sid, const voch_callhistory_t *match)
{
  VocHistoryInfo *info = NULL;
  guint32 *index = NULL;
  voch_cid_t **cids = NULL;
  voch_fid_t fid = HISTORY_CID;
  VocHistory *priv = NULL;
  Iterator *iter = NULL;
  gint count;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);
  if (index)
  {
    info = (VocHistoryInfo *) g_ptr_array_index (vochlists, *index);
  }
  g_assert (info);
  priv = (VocHistory *) info->priv;
  iter = iterator_new_full ((ElementFreeFunc) record_free);
  if (match)
  {
    if (record_db_get_records
        (priv->history_db, iter, match->record, 1, &fid) != DB_ERROR_NONE)
    {
      g_print ("%s(): record get failed.\n", __FUNCTION__);
      return NULL;
    }
  }

  else
  {
    if (record_db_get_records (priv->history_db, iter, NULL, 1, &fid) !=
        DB_ERROR_NONE)
    {
      g_print ("%s(): record get failed.\n", __FUNCTION__);
      return NULL;
    }
  }
  count = iterator_size (iter);
  if (count != 0)
  {
    gint i;

    cids = (voch_cid_t **) g_new0 (gpointer, count + 1);
    for (i = 0, iterator_to_first (iter);
         iterator_at_last (iter) != (gint) TRUE; i++)
    {
      Record *record = (Record *) iterator_remove_curr (iter);

      cids[i] = record_get_single_field (record, HISTORY_CID);
      record_free (record);
    }
    cids[i] = NULL;
  }

  else
  {
    cids = NULL;
  }
  iterator_free (iter);
  return (voch_cid_t **) cids;
}

voch_callhistory_t **
voch_get_historys (voch_sid_t sid, const voch_callhistory_t *match)
{
  VocHistoryInfo *info = NULL;
  guint32 *index = NULL;
  voch_callhistory_t **historys = NULL;
  VocHistory *priv = NULL;
  Iterator *iter = NULL;
  gint count;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);
  if (index)
  {
    info = (VocHistoryInfo *) g_ptr_array_index (vochlists, *index);
  }
  g_assert (info);
  priv = (VocHistory *) info->priv;
  iter = iterator_new_full ((ElementFreeFunc) record_free);
  if (match)
  {
    if (record_db_get_records
        (priv->history_db, iter, match->record, 0, NULL) != DB_ERROR_NONE)
    {
      g_print ("%s(): record get failed.\n", __FUNCTION__);
      return NULL;
    }
  }

  else
  {
    if (record_db_get_records (priv->history_db, iter, NULL, 0, NULL) !=
        DB_ERROR_NONE)
    {
      g_print ("%s(): record get failed.\n", __FUNCTION__);
      return NULL;
    }
  }
  count = iterator_size (iter);
  if (count != 0)
  {
    gint i;

    historys = (voch_callhistory_t **) g_new0 (gpointer, count + 1);
    for (i = 0, iterator_to_first (iter);
         iterator_at_last (iter) != (gint) TRUE; i++)
    {
      Record *record = (Record *) iterator_remove_curr (iter);

      historys[i] =
        voch_callhistory_new (record_db_get_template (priv->history_db));
      voch_callhistory_set_record (historys[i], record);
    }
    historys[i] = NULL;
  }

  else
  {
    historys = NULL;
  }
  iterator_free (iter);
  return (voch_callhistory_t **) historys;
}

//Get all the history items with the type from the database,
//create and return "voch_callhistory_t" for each items.
voch_callhistory_t **
voch_get_historys_by_type (voch_sid_t sid, voch_call_type_t type)
{
  voch_callhistory_t *match;
  voch_callhistory_t **result;

  if (voch_create_history (sid, &match) != VOCH_ERR_NONE)
  {
    g_print ("%s():  create history error\n", __FUNCTION__);
    return NULL;
  }
  voch_callhistory_set_type (match, type);
  result = voch_get_historys (sid, match);
  voch_callhistory_free (match);
  return result;
}

voch_fid_t **
voch_get_field_templates (voch_sid_t sid)
{
  VocHistoryInfo *info = NULL;
  guint32 *index = NULL;
  voch_fid_t **ftids = NULL;
  VocHistory *priv = NULL;
  Iterator *voc_ftids = NULL;
  gint count;

  index = (guint32 *) g_hash_table_lookup (session_maps, &sid);
  if (index)
  {
    info = (VocHistoryInfo *) g_ptr_array_index (vochlists, *index);
    g_assert (info);
    priv = (VocHistory *) info->priv;
    voc_ftids =
      field_template_get_fids (record_db_get_template (priv->history_db));
    count = iterator_size (voc_ftids);
    if (count != 0)
    {
      gint i;

      ftids = (voch_fid_t **) g_new0 (gpointer, count + 1);
      for (i = 0, iterator_to_first (voc_ftids);
           iterator_at_last (voc_ftids) != (gint) TRUE; i++)
      {
        voch_fid_t *ftid = (voch_ftid_t *) iterator_remove_curr (voc_ftids);

        ftids[i] = g_new0 (voch_fid_t, 1);
        memcpy (ftids[i], ftid, sizeof (voch_fid_t));
        g_free (ftid);
      }
      ftids[i] = NULL;
    }

    else
    {
      ftids = NULL;
    }
    iterator_free (voc_ftids);
    return (voch_fid_t **) ftids;
  }
  return ftids;
}

