/* 	libchenabler - API definition of the call history service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <glib.h>

#include <record.h>
#include <template.h>
#include <field.h>
#include "vochcallhistory.h"
#include "vochtemplate.h"
#include "vochcallhistory-priv.h"

voch_callhistory_t *
voch_callhistory_new (const FieldTemplate *template)
{
  voch_callhistory_t *history = NULL;

  g_assert (template);

  history = g_new0 (voch_callhistory_t, 1);
  if (history == NULL)
  {
    return NULL;
  }

  history->record = record_new (template);
  if (history->record == NULL)
  {
    g_free (history);

    return NULL;
  }

  return history;
}

voch_err_t
voch_callhistory_free (voch_callhistory_t *history)
{
  if (history == NULL)
  {
    return VOCH_ERR_NONE;
  }

  if (history->record)
  {
    record_free (history->record);
  }

  g_free (history);

  return VOCH_ERR_NONE;
}

voch_err_t
voch_callhistory_set_field (voch_callhistory_t *history, voch_fid_t fid,
                            const void *value)
{
  g_assert (history);
  g_assert (value);

  if (fid >= FIELD_ID_USER_DEFINED)
  {
    guint32 major = FIELD_ID_INVALID;
    gint index = 0;

    major = fid / VOC_FIELD_ID_DUP_RANGE;
    index = fid % VOC_FIELD_ID_DUP_RANGE;

    record_set_multiple_field (history->record, major, index, value);
  }
  else
  {
    if (record_is_multiple_field (history->record, fid) == TRUE)
    {
      record_set_multiple_field (history->record, fid, 0, value);
    }
    else
    {
      record_set_single_field (history->record, fid, value);
    }
  }
  return VOCH_ERR_NONE;
}

voch_err_t
voch_callhistory_get_field (voch_callhistory_t *history, voch_fid_t fid,
                            void **value)
{
  g_assert (history);
  g_assert (value);

  if (fid >= FIELD_ID_USER_DEFINED)
  {
    guint32 major = FIELD_ID_INVALID;
    gint index = 0;

    major = fid / VOC_FIELD_ID_DUP_RANGE;
    index = fid % VOC_FIELD_ID_DUP_RANGE;

    *value = record_get_multiple_field (history->record, major, index);
  }
  else
  {
    if (record_is_multiple_field (history->record, fid) == TRUE)
    {
      *value = record_get_multiple_field (history->record, fid, 0);
    }
    else
    {
      *value = record_get_single_field (history->record, fid);
    }
  }

  return VOCH_ERR_NONE;
}

voch_err_t
voch_callhistory_set_record (voch_callhistory_t *history,
                             const Record *record)
{
  if (record == NULL)
  {
    return VOCH_ERR_COMMON_NULL;
  }

  if (history->record)
  {
    record_free (history->record);
  }

  history->record = record;

  return VOCH_ERR_NONE;
}

void
voch_callhistory_print (const voch_callhistory_t *history)
{
  if (history == NULL)
  {
    return;
  }

  g_print ("History:[0x%08x]\n", (uint32_t) history);
  record_print (history->record);

  return;
}

voch_err_t
voch_callhistory_get_cid (const voch_callhistory_t *history, voch_cid_t *cid)
{
  return voch_callhistory_get_field (history, HISTORY_CID, (void **) cid);
}

voch_err_t
voch_callhistory_get_type (const voch_callhistory_t *history,
                           voch_call_type_t **type)
{
  return voch_callhistory_get_field (history, HISTORY_TYPE, (void **) type);
}

voch_err_t
voch_callhistory_get_date (const voch_callhistory_t *history, time_t **date)
{
  return voch_callhistory_get_field (history, HISTORY_DATE, (void **) date);
}

voch_err_t
voch_callhistory_get_duration (const voch_callhistory_t *history,
                               time_t **duration)
{
  return voch_callhistory_get_field (history, HISTORY_DURATION,
                                     (void **) duration);
}

voch_err_t
voch_callhistory_get_phonenumber (const voch_callhistory_t *history,
                                  char **phonenumber)
{
  return voch_callhistory_get_field (history, HISTORY_PHONENUM,
                                     (void **) phonenumber);
}

voch_err_t
voch_callhistory_set_date (voch_callhistory_t *history, time_t date)
{
  return voch_callhistory_set_field (history, HISTORY_DATE, (void *) &date);
}

voch_err_t
voch_callhistory_set_duration (voch_callhistory_t *history, time_t duration)
{
  return voch_callhistory_set_field (history, HISTORY_DURATION,
                                     (void *) &duration);
}

voch_err_t
voch_callhistory_set_phonenumber (voch_callhistory_t *history,
                                  char *phonenumber)
{
  return voch_callhistory_set_field (history, HISTORY_PHONENUM,
                                     (void *) phonenumber);
}

voch_err_t
voch_callhistory_set_type (voch_callhistory_t *history, voch_call_type_t type)
{
  return voch_callhistory_set_field (history, HISTORY_TYPE, (void *) &type);
}

